@@ -1005,7 +1005,11 @@ int CommandFunction::calculate(MathStructure &mstruct, const MathStructure &varg
 		output += buffer;
 	}
 
+#ifdef _WIN32
+	if(_pclose(pipe) > 0 && output.empty()) {
+#else
 	if(pclose(pipe) > 0 && output.empty()) {
+#endif
 		CALCULATOR->error(true, _("Failed to run external command (%s)."), commandline.c_str(), NULL);
 		return 0;
 	}
@@ -1098,6 +1102,14 @@ PlotFunction::PlotFunction() : MathFunction("plot", 1, -1) {
 	setDescription(str);
 }
 
+int PlotFunction::parse(MathStructure &mstruct, const std::string &eq, const ParseOptions &po) {
+	int ret = MathFunction::parse(mstruct, eq, po);
+	if(mstruct.size() > 0 && mstruct[0].isComparison() && mstruct[0].comparisonType() == COMPARISON_EQUALS && mstruct[0][0].isVariable() && mstruct[0][0].variable() == CALCULATOR->getVariableById(VARIABLE_ID_Y) && mstruct[0][1].contains(CALCULATOR->getVariableById(VARIABLE_ID_X))) {
+		mstruct[0].setToChild(2, true);
+	}
+	return ret;
+}
+
 int PlotFunction::calculate(MathStructure &mstruct, const MathStructure &vargs, const EvaluationOptions &eo) {
 
 	EvaluationOptions eo2;
@@ -1302,7 +1314,9 @@ int PlotFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 		eo2.expand = eo.expand;
 	}
 	if(mstruct.isFunction() && (mstruct.function()->id() == FUNCTION_ID_HORZCAT || mstruct.function()->id() == FUNCTION_ID_VERTCAT)) mstruct.setType(STRUCT_VECTOR);
-	if(mstruct.isMatrix() && mstruct.columns() == 1 && mstruct.rows() > 1) mstruct.transposeMatrix();
+	if(mstruct.isMatrix() && mstruct.columns() == 1 && mstruct.rows() > 1) {
+		if(!mstruct.transposeMatrix()) return 0;
+	}
 	CALCULATOR->endTemporaryStopIntervalArithmetic();
 	vector<MathStructure> x_vectors, y_vectors;
 	vector<PlotDataParameters*> dpds;
